<?php

namespace Drupal\book_moderation_sync\Hook;

use Drupal\book\BookOutlineStorageInterface;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\node\NodeInterface;

/**
 * Hooks for book moderation.
 */
class BookModerationHooks {

  protected LoggerChannelInterface $logger;

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ModerationInformationInterface $moderationInformation,
    protected BookOutlineStorageInterface $bookOutlineStorage,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->logger = $logger_factory->get('book_moderation_sync');
  }

  #[Hook('node_presave')]
  public function nodePresave(NodeInterface $entity) {
    if ($this->moderationInformation->isModeratedEntity($entity) && $entity->id()) {
      $this->syncModerationState($entity, $entity->get('moderation_state')->value);
    }
    else {
      $this->logWarning('Node @label does not have an ID yet. Skipping moderation state sync.', [
        '@label' => $entity->label(),
      ]);
    }
  }

  protected function syncModerationState(NodeInterface $node, $state) {
    foreach ($this->loadBookChildren($node) as $child_entity) {
      if ($this->moderationInformation->isModeratedEntity($child_entity)) {
        $child_entity->set('moderation_state', $state);
        try {
          $child_entity->save();
        }
        catch (EntityStorageException $e) {
          $this->logError('Failed to sync moderation state for child node @nid: @message', [
            '@nid' => $child_entity->id(),
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  public function deleteBookChildren(NodeInterface $node) {
    foreach ($this->loadBookChildren($node) as $child_entity) {
      $this->logDebug('Deleting child node @nid of book node @parent_nid.', [
        '@nid' => $child_entity->id(),
        '@parent_nid' => $node->id(),
      ]);

      try {
        $child_entity->delete();
        $this->logDebug('Child node @nid deleted successfully.', [
          '@nid' => $child_entity->id(),
        ]);
      }
      catch (EntityStorageException $e) {
        $this->logError('Failed to delete child node @nid: @message', [
          '@nid' => $child_entity->id(),
          '@message' => $e->getMessage(),
        ]);
      }
    }
  }

  private function loadBookChildren(NodeInterface $node): \Generator {
    if (!$node->id()) {
      $this->logError('Cannot load children for a node without an ID.');
      return;
    }

    $result = $this->bookOutlineStorage->loadBookChildren($node->id());
    if (empty($result)) {
      $this->logDebug('No children found for book node @nid.', [
        '@nid' => $node->id(),
      ]);
      return;
    }

    foreach ($result as $child_id => $data) {
      $child_entity = $this->entityTypeManager->getStorage('node')->load($child_id);
      if ($child_entity) {
        yield $child_entity;
      }
    }
  }

  private function logDebug(string $message, array $context = []) {
    $this->logger->debug($message, $context);
  }

  private function logError(string $message, array $context = []) {
    $this->logger->error($message, $context);
  }

  private function logWarning(string $message, array $context = []) {
    $this->logger->warning($message, $context);
  }
}
