<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\content_moderation\Traits\ContentModerationTestTrait;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  use ContentModerationTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Active le module Content Moderation.
    \Drupal::service('module_installer')->install(['content_moderation']);

    // Créez le type de contenu "chapter".
    $this->drupalCreateContentType(['type' => 'chapter', 'name' => 'Chapter']);

    // Configurez le type de contenu "chapter" comme un livre.
    $config = $this->config('book.settings');
    $allowed_types = $config->get('allowed_types');
    $allowed_types[] = 'chapter';
    $config->set('allowed_types', $allowed_types)->save();

    // Créez le workflow éditorial.
    $workflow = $this->createEditorialWorkflow();
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'book');
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'chapter');
    $workflow->save();

    // Charge l'utilisateur avec l'ID 1 (l'administrateur initial).
    $this->adminUser = \Drupal\user\Entity\User::load(1);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests that moderation states are synchronized between books and chapters.
   */
  public function testModerationStateSync() {
    // Create a book.
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);

    // Create a chapter and add it to the book.
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'book' => ['bid' => $book->id()],
      'moderation_state' => 'draft',
    ]);

    // Publish the book.
    $book->set('moderation_state', 'published');
    $book->save();

    // Reload the chapter and check its moderation state.
    $chapter = Node::load($chapter->id());
    $this->assertEquals('published', $chapter->get('moderation_state')->value, 'Chapter moderation state is synchronized with the book.');
  }

  /**
   * Tests deletion of child nodes when a book is deleted.
   */
  public function testDeleteBookChildren() {
    // Create a book.
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);

    // Create a chapter and add it to the book.
    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'book' => ['bid' => $book->id()],
      'moderation_state' => 'draft',
    ]);

    // Enable the "Delete children when a book is deleted" option.
    $this->config('book_moderation_sync.settings')
      ->set('delete_children_on_book_delete', TRUE)
      ->save();

    // Delete the book node.
    $book->delete();

    // Ensure the chapter node is deleted.
    $this->assertNull(Node::load($chapter->id()), 'Chapter node was deleted when the book was deleted.');
  }

}
